//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_Input_Library, Example4 - Joystick
//
// Date: 13th January, 2004
// Author(s): Darren John Dwyer, Brian Austin
// Copyright: (c) 2002-2004 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This demo shows how to control a force-feedback joystick.
// It enumerates devices, allows you to retrieve a devices capabilities,
// create an interface to a force feedback joystick, create a force-feedback
// effect and send it to the joystick.
//---------------------------------------------------------------------------
#ifndef TMainFormH
#define TMainFormH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
#include <vcl\Controls.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Forms.hpp>
#include "TDx_Input.h"
#include "TDx_Input_Library_Install.h"
#include <vcl\Buttons.hpp>
#include "TDx_InputDevice.h"
#include "TDIDevCaps.h"
#include "tdx_inputeffect.h"
#include "tdidataformat.h"
#include "tdijoystate2.h"
#include "tdipropdword.h"
#include "tdipropheader.h"
#include "tdiperiodic.h"
#include "tdienvelope.h"
#include "tdieffect.h"
#include "tdidevcaps.h"
#include "tdx_input.h"
#include "tdx_input_library_functions.h"
#include "tdx_inputdevice.h"
#include "tdiconstantforce.h"
#include "tdieffectinfo.h"
#include <vcl\ExtCtrls.hpp>
//---------------------------------------------------------------------------

#if (__BORLANDC__ < 0x0530) // BCB Version 1
class TMainForm : public TForm
{
#else // BCB Version 3+
class PACKAGE TMainForm : public TForm
{
#endif

__published:
//---------------------------------------------------------------------------
// Anything that is to be displayed within the Object Inspector
//---------------------------------------------------------------------------

    TDx_Input *Dx_Input1;
    TListBox *DevicesListBox;
    TLabel *Label2;
    TLabel *Label1;
    TMemo *InfoMemo;
    TLabel *Label5;
    TMemo *StatusMemo;
    TBitBtn *ClearMessagesButton;
    TDx_InputDevice *Dx_InputDevice1;
    TListBox *DeviceCapsListBox;
    TLabel *Label3;
    TDIDevCaps *DIDevCaps1;
    TDx_InputEffect *Dx_InputEffect1;
    TListBox *EffectsListBox;
    TLabel *Label4;
    TDIEffect *DIEffect2;
    TDIConstantForce *DIConstantForce1;
    TBitBtn *CreateEffectButton;
    TBitBtn *PlayEffectButton;
    TBitBtn *StopEffectButton;
    TDIPeriodic *DIPeriodic1;
    TDIEffect *DIEffect1;
    TDIEnvelope *DIEnvelope1;
    TDIPropDWORD *DIPropDWORD1;
    TDIPropHeader *DIPropHeader1;
	TDIJoyState2 *DIJoyState2;
	TRadioGroup *EffectDirection;
	TRadioGroup *EffectButton;
	TBitBtn *DestroyEffectButton;
	TBitBtn *ExitButton;
	TLabel *Label6;
	TListBox *ObjectsListBox;
    void __fastcall FormCreate(TObject *Sender);
    void __fastcall FormDestroy(TObject *Sender);
    
    void __fastcall ExitButtonClick(TObject *Sender);
    void __fastcall ClearMessagesButtonClick(TObject *Sender);
    
    void __fastcall Dx_Input1EnumDevices(TObject *Sender,
    TDIDeviceInstance *DeviceInstance, void *Context, bool &Finished);
    void __fastcall Dx_Input1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage);
    
    void __fastcall Dx_Input1Create(TObject *Sender);
    
    void __fastcall DevicesListBoxClick(TObject *Sender);
    
    void __fastcall Dx_Input1Destroy(TObject *Sender);
    void __fastcall Dx_InputDevice1Create(TObject *Sender);
    void __fastcall Dx_InputDevice1Destroy(TObject *Sender);
    
    
    
    
    void __fastcall Dx_InputDevice1EnumEffects(TObject *Sender, TDIEffectInfo *EffectInfo, void *Context,
    bool &Finished);
    void __fastcall CreateEffectButtonClick(TObject *Sender);
    
    
    void __fastcall StopEffectButtonClick(TObject *Sender);
    
    

	
	
	
	

	
	
	void __fastcall PlayEffectButtonClick(TObject *Sender);
	void __fastcall DestroyEffectButtonClick(TObject *Sender);
	
	
	void __fastcall FormShow(TObject *Sender);
	
	
	

	void __fastcall Dx_InputDevice1EnumObjects(TObject *Sender,
	TDIDeviceObjectInstance *ObjectInstance, void *Context, bool &Finished);
public:
//---------------------------------------------------------------------------
// Anything available to other classes and/or code
//---------------------------------------------------------------------------

    __fastcall TMainForm(TComponent* Owner);
    virtual __fastcall ~TMainForm();

protected:
//---------------------------------------------------------------------------
// Anything available only to classes descended from this class
//---------------------------------------------------------------------------

private:
//---------------------------------------------------------------------------
// Private data not accessible except from within this class
//---------------------------------------------------------------------------

    #define MAX_DEVICES 5

    dword       DevTypes[MAX_DEVICES];
    GUID        FFDriverGuids[MAX_DEVICES];
    GUID        InstanceGuids[MAX_DEVICES];
    AnsiString  InstanceNames[MAX_DEVICES];
    GUID        ProductGuids[MAX_DEVICES];
    AnsiString  ProductNames[MAX_DEVICES];
    word        Usages[MAX_DEVICES];
    word        UsagePages[MAX_DEVICES];
    int         NumDevices;

    #define MAX_EFFECTS 25

    dword       EffectDynamicParams[MAX_EFFECTS];
    dword       EffectTypes[MAX_EFFECTS];
    GUID        EffectGuids[MAX_EFFECTS];
    AnsiString  EffectNames[MAX_EFFECTS];
    dword       EffectStaticParams[MAX_EFFECTS];
    int         NumEffects;

    #define MAX_OBJECTS 25

    AnsiString	ObjectNames[MAX_OBJECTS];
    dword		ObjectIDs[MAX_OBJECTS];
    dword		ObjectOffsets[MAX_OBJECTS];
    int         NumObjects;

};
//---------------------------------------------------------------------------
#if (__BORLANDC__ < 0x0530) // BCB Version 1
extern TMainForm *MainForm;
#else // BCB Version 3+
extern PACKAGE TMainForm *MainForm;
#endif
//---------------------------------------------------------------------------
#endif
