//---------------------------------------------------------------------------
// File: TMainForm.h
//
//  TDx_Input_Library
//  Example6- Force Feedback Joysticks
//
// Date: 13th January, 2004
// Author: Kerrigan Burgess, Darren Dwyer
// Copyright: (c) 2001 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to control a Force Feedback Joystick,
// (or other Force Feedback Device)
//
// It helps significantly if you view and understand the previous DirectInput
// examples before you try to understand this demo.
//
//---------------------------------------------------------------------------
#ifndef TMainFormH
#define TMainFormH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
#include <vcl\Controls.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Forms.hpp>
#include "TDx_Input.h"
#include "TDx_Input_Library_Install.h"
#include <vcl\Buttons.hpp>
#include "TDx_InputDevice.h"
#include "TDIDevCaps.h"
#include "tdx_inputeffect.h"
#include "tdidataformat.h"
#include "tdijoystate2.h"
#include "tdipropdword.h"
#include "tdipropheader.h"
#include "tdiperiodic.h"
#include "tdienvelope.h"
#include "tdieffect.h"
#include "TDIEffectInfo.h"
#include "tdidevcaps.h"
#include "tdx_input.h"
#include "tdx_input_library_defns.h"
#include "tdx_inputdevice.h"
#include "tdiconstantforce.h"
#include "tdifileeffect.h"
#include <Dialogs.hpp>
#include <Grids.hpp>
#include <ComCtrls.hpp>
#include <CheckLst.hpp>
#include "tdirampforce.h"
#include <ExtCtrls.hpp>
#include "tdieffectinfo.h"
//---------------------------------------------------------------------------

// helper structures
struct	TDevice
{
    dword       DevTypes;
    GUID        FFDriverGuids;
    GUID        InstanceGuids;
    AnsiString  InstanceNames;
    GUID        ProductGuids;
    AnsiString  ProductNames;
    word        Usages;
    word        UsagePages;
};

struct TEffect
{
    GUID		Guid;
    AnsiString  Name;
    dword 		Type;
};

struct TCreated
{
	struct TDataNode
    {
	    TDx_InputEffect*	InputEffect;
        AnsiString			Name;
        bool				InUse;
    };

    TDataNode** 		Data;
    int					Num;

    TCreated() { Data = NULL; Num = 0; }
    ~TCreated() { Clear(); }

    int Add(void)
    {
    	TDataNode** tmp = new TDataNode *[Num+1];
        memset(tmp, 0, Num+1);
        for (int i=0;i<Num;i++)
        	tmp[i] = Data[i];

        delete[] Data;
        Data = tmp;
        Num++;

		// now allocate space for the data node
        Data[Num-1] = new TDataNode;

        return Num-1;
    }

    void Clear(void)
    {
    	for (int i=0;i<Num;i++)
        {
        	delete Data[i]->InputEffect;
            delete Data[i];
        }
		delete[] Data;
    }

    bool Remove(int Index)
    {
    	if (Index<Num)
        {
        	delete Data[Index]->InputEffect;
            delete Data[Index];

        	if (Num>1)
            {
        		TDataNode** tmp = new TDataNode *[Num-1];
                memset(tmp, 0, Num-1);

                int i=0;
                for (;i<Index;i++)
                	tmp[i] = Data[i];

                for (int j=(Index+1);j<Num;j++)
                	tmp[i] = Data[j];

				delete[] Data;
                Data = tmp;
                Num--;
            }
            else
            {
            	delete[] Data;
                Data = NULL;
                Num = 0;
            }
            return true;
        }
        else
        	return false;
    }
};

#if (__BORLANDC__ < 0x0530) // BCB Version 1
class TMainForm : public TForm
{
#else // BCB Version 3+
class PACKAGE TMainForm : public TForm
{
#endif

__published:
//---------------------------------------------------------------------------
// Anything that is to be displayed within the Object Inspector
//---------------------------------------------------------------------------

    TDx_Input *Dx_Input1;
    TBitBtn *EnumerateButton;
	TListBox *DeviceListBox;
    TLabel *Label2;
    TLabel *Label1;
    TMemo *InfoMemo;
    TLabel *Label5;
    TMemo *StatusMemo;
    TBitBtn *ClearMessagesButton;
    TBitBtn *ExitButton;
    TBitBtn *CreateDefaultButton;
    TBitBtn *CreateSelectedButton;
    TDx_InputDevice *Dx_InputDevice1;
    TListBox *DeviceCapsListBox;
    TLabel *Label3;
    TBitBtn *RetrieveCapsButton;
    TDIDevCaps *DIDevCaps1;
    TBitBtn *DestroyActiveButton;
    TBitBtn *DestroyInterfaceButton;
    TDx_InputEffect *Dx_InputEffect1;
    TDIJoyState2 *DIJoyState21;
    TDIPropDWORD *DIPropDWORD1;
    TDIPropHeader *DIPropHeader1;
    TBitBtn *CreateEffectButton;
    TDIPeriodic *DIPeriodic1;
    TDIEnvelope *DIEnvelope1;
    TDIEffect *DIEffect1;
    TBitBtn *EnumEffectsButton;
    TBitBtn *PlayEffectButton;
    TBitBtn *StopEffectButton;
    TListBox *EffectsListBox;
    TLabel *Label4;
	TDIConstantForce *DIConstantForce1;
	TBitBtn *LoadEffectButton;
	TLabel *Label6;
	TDIFileEffect *DIFileEffect1;
	TOpenDialog *OpenDialog1;
	TGroupBox *df;
	TStaticText *StaticText1;
	TStaticText *StaticText2;
	TComboBox *EffectAxesInterpComboBox;
	TStaticText *StaticText3;
	TStaticText *StaticText4;
	TEdit *EffectDurationEdit;
	TStaticText *StaticText5;
	TEdit *EffectGainEdit;
	TStaticText *StaticText6;
	TEdit *EffectSampleEdit;
	TStaticText *StaticText7;
	TEdit *EffectDelayEdit;
	TCheckListBox *AttachedEffectsListBox;
	TComboBox *CreatedEffectsComboBox;
	TLabel *Label7;
	TDIRampForce *DIRampForce1;
	TGroupBox *GroupBox1;
	TStaticText *StaticText18;
	TEdit *EnvelopeAttackLevelEdit;
	TStaticText *StaticText19;
	TEdit *EnvelopeAttackTimeEdit;
	TStaticText *StaticText20;
	TEdit *EnvelopeFadeLevelEdit;
	TStaticText *StaticText21;
	TEdit *EnvelopeFadeTimeEdit;
	TEdit *EffectsTypeEdit;
	TGroupBox *GroupBox2;
	TNotebook *EffectTypeNotebook;
	TStaticText *StaticText22;
    TEdit *ConstantMagnitudeEdit;
	TStaticText *StaticText23;
    TEdit *PeriodicMagnitudeEdit;
	TStaticText *StaticText24;
    TEdit *PeriodicOffsetEdit;
	TStaticText *StaticText25;
    TEdit *PeriodicPeriodEdit;
	TStaticText *StaticText26;
    TEdit *PeriodicPhaseEdit;
	TStaticText *StaticText27;
    TEdit *RampStartEdit;
    TEdit *RampEndEdit;
	TStaticText *StaticText28;
	TStaticText *StaticText29;
	TStaticText *StaticText30;
    TCheckListBox *EffectAxesCheckListBox;
    TEdit *EffectDirectionXEdit;
    TEdit *EffectDirectionYEdit;
	TStaticText *StaticText8;
	TDIEffectInfo *DIEffectInfo1;
	TCheckBox *CreatedEffectsAttachedCheckBox;
	TBitBtn *SaveEffectButton;
	TSaveDialog *SaveDialog1;
	TBitBtn *UpdateEffectButton;
    void __fastcall FormCreate(TObject *Sender);
    void __fastcall FormDestroy(TObject *Sender);
    void __fastcall EnumerateButtonClick(TObject *Sender);
    void __fastcall ExitButtonClick(TObject *Sender);
    void __fastcall ClearMessagesButtonClick(TObject *Sender);

    void __fastcall Dx_Input1EnumDevices(TObject *Sender,
    TDIDeviceInstance *DeviceInstance, void *Context, bool &Finished);
    void __fastcall Dx_Input1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage);
    void __fastcall CreateDefaultButtonClick(TObject *Sender);
    void __fastcall Dx_Input1Create(TObject *Sender);

    void __fastcall DeviceListBoxClick(TObject *Sender);
    void __fastcall CreateSelectedButtonClick(TObject *Sender);
    void __fastcall Dx_Input1Destroy(TObject *Sender);
    void __fastcall Dx_InputDevice1Create(TObject *Sender);
    void __fastcall Dx_InputDevice1Destroy(TObject *Sender);
    void __fastcall RetrieveCapsButtonClick(TObject *Sender);
    void __fastcall DestroyActiveButtonClick(TObject *Sender);
    void __fastcall DestroyInterfaceButtonClick(TObject *Sender);

    void __fastcall Dx_InputDevice1EnumEffects(TObject *Sender, TDIEffectInfo *EffectInfo, void *Context,
    bool &Finished);
	void __fastcall Dx_InputDeviceEnumEffectsInFile(TObject *Sender, TDIFileEffect *FileEffect, void *Context, bool &Finished);
    void __fastcall CreateEffectButtonClick(TObject *Sender);
    void __fastcall EnumEffectsButtonClick(TObject *Sender);
    void __fastcall PlayEffectButtonClick(TObject *Sender);
    void __fastcall StopEffectButtonClick(TObject *Sender);
    void __fastcall EffectsListBoxClick(TObject *Sender);
	void __fastcall LoadEffectButtonClick(TObject *Sender);
	void __fastcall CreatedEffectsComboBoxClick(TObject *Sender);
	void __fastcall AttachedEffectsListBoxClick(TObject *Sender);
	void __fastcall AttachedEffectsListBoxClickCheck(TObject *Sender);
	void __fastcall SaveEffectButtonClick(TObject *Sender);
	void __fastcall UpdateEffectButtonClick(TObject *Sender);
public:
//---------------------------------------------------------------------------
// Anything available to other classes and/or code
//---------------------------------------------------------------------------

    __fastcall TMainForm(TComponent* Owner);
    virtual __fastcall ~TMainForm();

protected:
//---------------------------------------------------------------------------
// Anything available only to classes descended from this class
//---------------------------------------------------------------------------

private:
//---------------------------------------------------------------------------
// Private data not accessible except from within this class
//---------------------------------------------------------------------------
	void __fastcall DeviceListBoxClear(void);
	void __fastcall EffectsListBoxClear(void);
	void __fastcall ResetEffectParameter(DWORD Type);
	void __fastcall UpdateEffectParameter(void);
	TDIEffect* __fastcall CreateFileEffect(int index);

	DWORD rgdwAxes[32];		// upto 32 axes may be in effect (Getparameters will set the number of axes in actual use)
    LONG rglDirection[32];
};
//---------------------------------------------------------------------------
#if (__BORLANDC__ < 0x0530) // BCB Version 1
extern TMainForm *MainForm;
#else // BCB Version 3+
extern PACKAGE TMainForm *MainForm;
#endif
//---------------------------------------------------------------------------
#endif
